/*
 * This file is part of JAMAL.
 *
 * JAMAL is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2, as published
 * by the Free Software Foundation.
 *
 * JAMAL is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JAMAL; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.jamal.client;


import com.jamal.JamalException;
import com.jamal.MatlabCaller;

import java.rmi.Naming;
import java.rmi.RemoteException;

/**
 * Client for calling Matlab functions from a Java program
 *
 * @version 1.0 Jan 17, 2010 3:03:17 PM
 * @author: Hut
 */
public class MatlabClient {

    private MatlabCaller matlabCaller;

    /**
     * Default constructor
     */
    public MatlabClient() throws JamalException {
        this(MatlabCaller.SERVER_ADDRESS, MatlabCaller.SERVER_PORT);
    }

    public MatlabClient(String host, int port) throws JamalException {
        try {
            matlabCaller = (MatlabCaller) Naming.lookup(String.format("rmi://%s:%d/%s", host, port, MatlabCaller.SERVICE_NAME));
        } catch (Exception e) {
            throw new JamalException(String.format("Matlab host seems to be dead host:%s port:%s", host, port), e);
        }
    }

    /**
     * Executes Matlab function and returns an array of output arguments
     * //TODO Send Matlab console stream to java console
     *
     * @param matlabFunctionName Matlab function name
     * @param inputArgs          Array of input arguments
     * @param numberOfOutputArgs Number of output arguments
     * @return output arguments as array of Object
     * @throws com.jamal.JamalException JamalException
     */
    public Object[] executeMatlabFunction(String matlabFunctionName, Object[] inputArgs, int numberOfOutputArgs) throws JamalException {

        Object[] output = null;
        try {
            output = matlabCaller.executeMatlabFunction(matlabFunctionName, inputArgs, numberOfOutputArgs);
        } catch (RemoteException e) {
            throw new JamalException("Error occured while executing Matlab function " + matlabFunctionName, e);
        }
        return output;
    }

    /**
     * Shuts down MatlabServer
     * @throws JamalException exception thrown
     */
    public void shutDownServer() throws JamalException {
        executeMatlabFunction("quit", null, 0);
    }

}
